import requests
import click


nes_url = 'http://nes.sktai.io/v1/runs'


def nes_submit(
    input_notebook,
    parameters=None,
):
    data = {}
    data['input_url'] = input_notebook
    if parameters:
        data['parameters'] = parameters
    res = requests.post(nes_url, json=data)
    print(f'Job submitted with: {data}')
    res.raise_for_status()
    id = res.json()['id']
    output_url = res.json()['output_url']
    print(f'id: {id}')
    print(f'output_url: {output_url}')
    return id, output_url


def nes_get_status(id):
    res = requests.get(f'{nes_url}/{id}')
    res.raise_for_status()
    status = res.json()['status']
    return status


def nes_execute(
    input_notebook,
    parameters=None,
):
    '''
    return: str
    Succeeeded or Failed or Error
    '''
    from time import sleep
    id, output_url = nes_submit(input_notebook, parameters=parameters)
    status = nes_get_status(id)
    poll_interval = 30
    while status in ['Running', 'Pending']:
        print(f'Polling job status... current status: "{status}"')
        sleep(poll_interval)
        status = nes_get_status(id)
    return status, output_url


@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.argument('input_notebook')
@click.option(
    '--parameters', '-p', nargs=2, multiple=True, help='Parameters to pass to the parameters cell.'
)
def nes_cli(
    input_notebook,
    parameters,
):
    parameters_final = {}
    for name, value in parameters or []:
        parameters_final[name] = value
    status, output_url = nes_execute(
        input_notebook,
        parameters=parameters_final,
    )
    print(f'output_url: {output_url}')
    print(f'status: {status}')



import requests
import json
from skt.vault_utils import get_secrets


def set_model_name(comm_db, params):
    secret = get_secrets('mls')
    if comm_db[-3:] == 'dev':  # stg
        url = secret['dev_url']
    else:  # prd
        url = secret['prd_url']
    requests.post(url, data=json.dumps(params))


def get_recent_model_path(comm_db, model_key):
    secret = get_secrets('mls')
    if comm_db[-3:] == 'dev':  # stg
        url = secret['dev_url']
    else:  # prd
        url = secret['prd_url']
    return requests.get(f'{url}/latest').json()[model_key]


def get_model_name(key):
    secret = get_secrets('mls')
    url = secret['prd_url']
    return requests.get(f'{url}/latest').json()[key]



from collections import Counter
from skt.ye import hive_to_pandas, slack_send


# post_filter 이후, context_mapping 을 마친 post_filter_with_context_id Table 생성
def context_mapping(
    meta_table=None,
    comm_db=None,
    reco_type=None,
    model_name=None,
    dt=None,
    feature_ym=None
):

    # 1. item_reco_predict_post_filter with impression 'Y' table LOAD
    query = f"""
    select *
    from {comm_db}.item_reco_predict_post_filter
    where reco_type = '{reco_type}'
    and model = '{model_name}'
    and dt = '{dt}'
    and impression_yn = 'Y'
    """
    post_filter_with_y = hive_to_pandas(query)
    print('------------------' + 'predict_post_filter' + ' WITH "Y"' + ' loaded')

    # 2. context_meta Table LOAD
    query = f"""
    select * from {meta_table}
    where item_id = '{post_filter_with_y['prod_id'][0]}'
    """
    meta = hive_to_pandas(query)
    print('------------------' + 'meta_table' + ' WITH "ITEM_ID (PROD_ID)"' + ' loaded')

    CONTEXT_NUM = meta.shape[0]
    CONTEXT_ID_DEFAULT = meta[meta['context_priority'] == 1]['context_id'].values[0]
    CONTEXT_ID_LIST = tuple(meta['context_id'].values)

    # 3. default_setting
    post_filter_with_y['context_id'] = CONTEXT_ID_DEFAULT
    print('------------------' + 'CONTEXT_NUM : 1 (DEFAULT)')

    if CONTEXT_NUM == 1:
        context_count_list = list(Counter(list(post_filter_with_y['context_id'].values)).items())
        msg = ('[CONTEXT-MAPPING-SUMMARY]' + '\n' + str(context_count_list[0][0]) + ' : ' + str(context_count_list[0][1]))

    # 4. context_num >= 2 : context mapping (with context priority)
    else:
        query = f"""
        select svc_mgmt_num, dimension
        from    comm.user_profile_derivative_monthly
        where   ym = '{feature_ym}'
        and     value = 'Y'
        and     dimension in {CONTEXT_ID_LIST}
        """
        df_context = hive_to_pandas(query)

        # 4+a. for loop - overwrite mapping
        for i in range(CONTEXT_NUM - 1):
            priority_num = i + 2
            print('------------------' + 'CONTEXT_NUM : ' + str(priority_num))
            PRIORITY_CONTEXT_ID = meta[meta['context_priority'] == priority_num]['context_id'].values[0]
            post_filter_with_y.loc[post_filter_with_y['svc_mgmt_num'].isin(list(df_context[df_context['dimension'] == PRIORITY_CONTEXT_ID]['svc_mgmt_num'])), "context_id"] = PRIORITY_CONTEXT_ID

        # 4+b. counter - context mapping statistics
        context_count_list = list(Counter(list(post_filter_with_y['context_id'].values)).items())
        msg = '[CONTEXT-MAPPING-SUMMARY]'
        for j in range(CONTEXT_NUM):
            msg_ = ('\n' + str(context_count_list[j][0]) + ' : ' + str(context_count_list[j][1]))
            msg += msg_

    # 5. Slack Report
    slack_send(text=msg, channel="#rec_modeling_alert", icon_emoji=':mag:')

    # 6. return post_filter_with_y_with_context_id Table
    return post_filter_with_y



import setuptools


def load_long_description():
    with open("README.md", "r") as f:
        long_description = f.read()
    return long_description


# Start dependencies group
air = [
    'bayesian-optimization>=1.1.0',
    'shap>=0.34.0',
    'gensim>=3.8.1',
    'torch>=1.4.0',
    'torchvision>=0.5.0',
    'seaborn>=0.10.0',
    'scikit-learn>=0.22.1',
    'lifelines>=0.24.0',
    'xgboost>=1.0.1',
    'lightgbm>=2.3.1',
    'implicit>=0.4.2',
    'tqdm==4.43.0',
    'matplotlib>=3.1.3',
]

setuptools.setup(
    name="skt",
    version="0.1.33",
    author="SKT",
    author_email="all@sktai.io",
    description="SKT package",
    long_description=load_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/sktaiflow/skt",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'thrift-sasl==0.3.0',
        'hvac>=0.9.6',
        'pyhive[hive]',
        'pyarrow>=0.16.0',
        'pandas>=1.0.1',
        'slackclient>=2.5.0',
        'google-cloud-bigquery>=1.24.0',
        'httplib2==0.17.0',
        'click',
        'PyGithub',
    ],
    entry_points={'console_scripts': ['nes = skt.nes:nes_cli']},
    extras_require={
        'air': air,
    }
)
